@@ -1,5 +1,6 @@
 """Tkinter GUI for pdf-slashannots"""
 import argparse
+from collections import Counter
 import importlib.resources as ir
 import os
 import os.path
@@ -14,7 +15,7 @@ from typing import *
 
 import PyPDF2
 
-from .main import PdfAnnotationRedacter, DatePrecision
+from .main import PdfAnnotationRedacter, AnnotationStats, DatePrecision
 
 
 class SlashAnnotsGUI(Tk):
@@ -41,7 +42,7 @@ class SlashAnnotsGUI(Tk):
         main_frame.pack(expand=True,fill=BOTH)
         # bottom controls
         bottom_frame = ttk.Frame(main_frame)
-        bottom_frame.pack(side=BOTTOM)
+        bottom_frame.pack(side=BOTTOM, pady=5)
         quit_btn = ttk.Button(bottom_frame, text="Quit", command=self.destroy)
         quit_btn.pack(side=RIGHT)
         redactall_btn = ttk.Button(bottom_frame, text="Redact all",
@@ -69,31 +70,56 @@ class SlashAnnotsGUI(Tk):
             text='Select PDF',
             command=self.select_pdf,
         )
-        open_button.pack(side=BOTTOM)
+        open_button.pack()
         # also offer shortcuts
         self.bind("<Control-o>", lambda e: self.select_pdf())
         self.bind("<Meta_L><o>", lambda e: self.select_pdf())
+        self.pdfstats_var = StringVar(value="test")
+        stats_label = ttk.Label(file_frame, textvariable=self.pdfstats_var,
+                                justify=CENTER)
+        stats_label.pack(side=BOTTOM)
 
         # - settings frame
         set_frame = ttk.Frame(left_frame)
         set_frame.pack(side=BOTTOM, pady=10)
         prec_label = ttk.Label(set_frame, text="Precision", padding=5)
-        prec_label.pack(side=LEFT)
+        prec_label.grid(row=1, column=1)
         prec_drop = ttk.Combobox(set_frame, width=10,
                                  values=[str(prec) for prec in DatePrecision],
                                  state="readonly")
-        prec_drop.pack(side=RIGHT)
+        prec_drop.grid(row=1, column=2)
         prec_drop.set("hour")
         self.prec_drop = prec_drop
+        self.rednames_chk = BooleanVar(value=False)
+        rednames_chk = ttk.Checkbutton(set_frame, text="Replace names with",
+                                       variable=self.rednames_chk)
+        rednames_chk.grid(row=2, column=1, columnspan=2)
+        self.redname_rpl = StringVar(value="unknown")
+        redname_entry = ttk.Entry(set_frame, width=20, justify=CENTER,
+                                  state=DISABLED,
+                                  textvariable=self.redname_rpl)
+        redname_entry.grid(row=3, column=1, columnspan=2)
+        def change_redname_state(*args) -> None:
+            del args  # not needed
+            if self.rednames_chk.get():
+                redname_entry["state"] = NORMAL
+            else:
+                redname_entry["state"] = DISABLED
+        self.rednames_chk.trace_add("write", change_redname_state)
 
         # name frame (right)
         name_frame = ttk.Frame(main_frame)
-        name_frame.pack(side=RIGHT)
+        name_frame.pack(side=RIGHT, padx=10)
         name_label = ttk.Label(name_frame, text="Select Author(s)")
         name_label.pack(side=TOP)
         yscroll = ttk.Scrollbar(name_frame, orient=VERTICAL)
         yscroll.pack(side=RIGHT, fill="y")
-        name_list = Listbox(name_frame, selectmode=MULTIPLE, border=0)
+        list_wrapper = Frame(name_frame, bg="white")
+        list_wrapper.pack(side=LEFT, fill=BOTH, expand=True)
+        name_list = Listbox(list_wrapper, selectmode=MULTIPLE, border=0,
+                            bg=list_wrapper.cget("bg"))
+        self.name_list = name_list
+        name_list.pack(fill=BOTH, expand=True, padx=5, pady=5)
         def name_list_update(event: Event):
             selection = event.widget.curselection()
             if selection:
@@ -101,9 +127,6 @@ class SlashAnnotsGUI(Tk):
             else:
                 self.disallow_selected()
         name_list.bind("<<ListboxSelect>>", name_list_update)
-
-        self.name_list = name_list
-        name_list.pack(side=LEFT, fill=BOTH, expand=True)
         name_list.config(yscrollcommand=yscroll.set)
         yscroll.config(command=name_list.yview)
 
@@ -129,12 +152,23 @@ class SlashAnnotsGUI(Tk):
     def set_pdf(self, pdfpath: Path):
         self.pdfpath = pdfpath
         self.file_label.config(text=self.pdfpath.name)
-        names = get_names(self.pdfpath)
+        names_ctr = get_names(self.pdfpath)
+        names = list(sorted(names_ctr.keys()))
         self.name_list.delete(0, END)  # clear box
         for i, name in enumerate(names):
             self.name_list.insert(i, name)
         # enable buttons
         self.redactall_btn["state"] = NORMAL
+        # update stats info
+        total = names_ctr.total()
+        if total > 0:
+            self.pdfstats_var.set(" ".join((
+                plural(total, "annotation"),
+                "from",
+                plural(len(names), "author"),
+            )))
+        else:
+            self.pdfstats_var.set("0 annotations")
 
     def allow_selected(self):
         self.redactsel_btn["state"] = NORMAL
@@ -156,28 +190,71 @@ class SlashAnnotsGUI(Tk):
         precision = self.prec_drop.get()
         redacter = PdfAnnotationRedacter(
             included_authors=authors,
+            redact_author=self.rednames_chk.get(),
+            redacted_author_name=self.redname_rpl.get(),
             precision=DatePrecision.argparse(precision),
         )
         outpath = self.pdfpath.with_suffix(".redacted.pdf")
         with self.pdfpath.open("rb") as pdffp, outpath.open("wb") as outfp:
             redacter.redact(pdffp, outfp)
-        statsout = io.StringIO()
-        redacter.stats.pprint_stats(statsout)
-        text = statsout.getvalue()
-        tkmsg.showinfo("Redaction successful", text, icon="info")
+        stats = redacter.stats
+        resview = ResultView(outpath, stats)
+        resview.mainloop()
 
 
-def get_names(pdffile: Path) -> List[str]:
+class ResultView(Toplevel):
+    def __init__(self, outpath: Path, stats: AnnotationStats, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+        self.title("Redaction result")
+        self.minsize(400, 200)
+        frame = ttk.Frame(self, padding=5)
+        frame.pack()
+        tree = ttk.Treeview(frame, columns=("annots", "nreds", "dreds"))
+        tree.pack(side=TOP, fill=BOTH, expand=True)
+        tree.column('annots', width=100, anchor='center')
+        tree.heading('annots', text='Annotations')
+        tree.column('nreds', width=100, anchor='center')
+        tree.heading('nreds', text='Redact. Names')
+        tree.column('dreds', width=100, anchor='center')
+        tree.heading('dreds', text='Redact. Dates')
+        # fill tree
+        authors = set(stats.authorship_ctr.keys())
+        for author in sorted(authors):
+            tree.insert("", END, text=author, values=(
+                stats.authorship_ctr[author],
+                stats.redacted_authorships[author],
+                stats.redacted_cdates[author],
+            ))
+        outframe = ttk.Frame(frame)
+        outpathlabel = ttk.Label(outframe, text=f"Output:")
+        outpathlabel.pack(side=LEFT)
+        outpathvar = StringVar(value=str(outpath))
+        outpathentry = Entry(outframe, state="readonly",
+                             width=80,
+                             textvariable=outpathvar)
+        outpathentry.pack(side=RIGHT, fill=X, expand=True)
+        outframe.pack()
+        closebtn = ttk.Button(frame, text="Close", command=self.destroy)
+        closebtn.pack(side=BOTTOM)
+
+
+def get_names(pdffile: Path) -> Counter[str]:
     reader = PyPDF2.PdfReader(pdffile)  # type: ignore
-    authors: Set[str] = set()
+    authors_ctr: Counter[str] = Counter()
     for page in reader.pages:
         if "/Annots" in page:
             for annot in page["/Annots"]:  # type: ignore
                 obj = annot.get_object()
                 if "/T" in obj:
                     author = obj["/T"]
-                    authors.add(author)
-    return list(sorted(authors))
+                    authors_ctr[author] += 1
+    return authors_ctr
+
+
+def plural(num: int, thing: str) -> str:
+    if num == 1:
+        return f"1 {thing}"
+    return f"{num} {thing}s"
 
 
 def main():
